import { Component } from 'solid-js';
import { peekCachedSignal } from '../../workers/cached-signals';
import RteIcon from '../../icons/directions-svgrepo-com.svg?component-solid';
import { useI18n } from '@solid-primitives/i18n';

import Tree from '../tree';
import { LineString } from 'ol/geom';
import DisplayOrGetAddress from '../display-or-get-address';
import { getFormatedLength } from '../../lib/ol';

interface Props {
  rteId: string;
}

const RteViewer: Component<Props> = ({ rteId }) => {
  const [t, { add, locale, dict }] = useI18n();

  const rte = peekCachedSignal({ id: rteId, action: 'getRte' });
  console.log({ caller: 'RteViewer', rteId, rte: rte() });
  const title = () => {
    return rte().name;
  };

  const lineString = new LineString(
    rte().rtept.map((rtept: any) => [rtept.$.lon, rtept.$.lat])
  );

  return (
    <Tree
      title={
        <>
          <RteIcon fill='black' width='24' height='24' /> {title()}
        </>
      }
      content={
        <>
          <div>
            {getFormatedLength(lineString)} {t('from')}{' '}
            <DisplayOrGetAddress
              target={() => rte().rtept.at(0)}
              putAction='putRtept'
              putParamName='rtept'
            />{' '}
            {t('to')}{' '}
            <DisplayOrGetAddress
              target={() => rte().rtept.at(-1)}
              putAction='putRtept'
              putParamName='rtept'
            />
          </div>
        </>
      }
      subTree={undefined}
    />
  );
};

export default RteViewer;
